#!/bin/sh

if [ "$1" = "" ]; then
	echo "Usage: BPMQueryDeploymentConfiguration <source install root> <source profile name> <snapshot directory> "
	exit
fi 

if [ "$1" = "-help" ]; then
	echo "Usage: BPMQueryDeploymentConfiguration <source install root> <source profile name> <snapshot directory> "
	exit
fi 

if [ ! -d "$1" ]; then
	echo "Source Install Directory $1 does not exist. Please specify a valid source install directory"
	exit
fi

if [ "$2" = "" ]; then
	echo "Profile name and snapshot directory not specified. Please specify a value for source profile name and snapshot directory." 
	echo "Usage: BPMQueryDeploymentConfiguration $1 <source profile name> <snapshot directory> "
	exit
fi

if [ "$3" = "" ]; then
	echo "Snapshot directory not specified." 
	echo "Usage: BPMQueryDeploymentConfiguration $1 $2 <snapshot directory>"
	exit
fi

echo "Parameters are $1 $2 $3"

if [ ! -d "$3"/profiles/$2 ]; then
	echo "Creating the folder"
	mkdir -p "$3"/profiles/$2
fi

MIGRATION_DIR="$3"
PROFILE_NAME="$2"

echo "Snapshot directory is $MIGRATION_DIR"
echo "Source profile name is $PROFILE_NAME"

WAS_USER_SCRIPT=""
WAS_HOME=""
USER_INSTALL_ROOT=""

binDir=`dirname "$0"`
. "$binDir"/setupCmdLine.sh
CURR_WAS_HOME="$WAS_HOME"

WAS_USER_SCRIPT=""
WAS_HOME=""
USER_INSTALL_ROOT=""

cd "$1"/bin

. "$1/bin"/setupCmdLine.sh -profileName "$2"

if [ "$USER_INSTALL_ROOT" = "" ]; then
      echo "Profile name $PROFILE_NAME specified is incorrect. Please specify a valid source profile name"
      echo "Usage: BPMQueryDeploymentConfiguration $1 <source profile name> $3"
      exit
fi

echo "Profile location is $USER_INSTALL_ROOT"

cd "$CURR_WAS_HOME"/bin

DATE=`date "+%a-%h-%d_%H.%M.%S-%Y"`
LOGFILENAME="$MIGRATION_DIR"/logs/BPMQueryDeploymentConfiguration_$DATE.log

if [ ! -d "$MIGRATION_DIR"/logs ]; then
	echo "Creating the logs directory"
	mkdir "$MIGRATION_DIR"/logs
fi

echo Executing "$USER_INSTALL_ROOT"/bin/wsadmin.sh -conntype NONE -lang jython -f "$CURR_WAS_HOME/util/upgrade/BPMQueryDeploymentConfiguration.py" -snapshotDir $MIGRATION_DIR -sourceProfileName $PROFILE_NAME
echo Please check the log file for any errors. The log file is "$LOGFILENAME"

"$USER_INSTALL_ROOT"/bin/wsadmin.sh -conntype NONE -lang jython -f "$CURR_WAS_HOME/util/upgrade/BPMQueryDeploymentConfiguration.py" -snapshotDir $MIGRATION_DIR -sourceProfileName $PROFILE_NAME > "$LOGFILENAME" 
