# BEGIN COPYRIGHT
# *************************************************************************
#
# Licensed Materials - Property of IBM
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# (C) Copyright IBM Corporation  2007, 2012. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *************************************************************************
# END COPYRIGHT

from java.lang import System
from com.ibm.wbicore.migration.profileupgrade import WBIProfileUpgradeHelper
from com.ibm.wsspi.scripting import ScriptingExtensionHelper
from com.ibm.wbicore.migration.internal.common import MigrationLogger;
from java.lang import Boolean;
from java.util.logging import Level;

import sys
import re
import sre

def regexp(pattern, string, flags=0):
	if(re.compile(pattern, flags).search(string)==None): return 0
	else: return 1
#endDef


#*******************************************************************************************************
# Procedure to query the connection status of the admin client ('CONNECTED', 'DISCONNECTED', or 'ERROR')
#*******************************************************************************************************
def getConnectionStatus (  ):
	global connectionStatus
	connectionStatus = "UNKNOWN"
	if not ScriptingExtensionHelper.getCurrentScriptingConfigService():
		connectionStatus = "ERROR"
	elif not ScriptingExtensionHelper.getCurrentScriptingAdminClient():
		connectionStatus = "DISCONNECTED"
	else:
		connectionStatus = "CONNECTED"
	#endif
	return connectionStatus
#endDef


#*******************************************************************************************************
# Procedure to check if a given cluster exists. Returns 1 if cluster exists or null/empty clusterName
#*******************************************************************************************************
def checkClusterName(clusterName):
	if (not clusterName):
		return 1
	if (AdminConfig.getid("/Cell:" + System.getProperty("local.cell") + "/ServerCluster:" + clusterName + "/")):
		return 1
	return 0
#endDef

#*******************************************************************************************************
# convert a string as returned by AdminConfig.show[Attribute] to a Jython list
#*******************************************************************************************************
def wsadminToList(inStr):
	outList=[]
	if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
		inStr = inStr[1:-1]
		tmpList = inStr.split(" ")
	else:
		tmpList = inStr.split("\n")  #splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip();        #removes any Windows "\r"
		if (len(item)>0):
			outList.append(item)
	return outList
#endDef

#*******************************************************************************************************
# Procedure to invoke a given callable on each deployment target of a given set of applications
#*******************************************************************************************************
def updateDeploymentTargetsOf(appNames, updateDeploymentTarget):
	# determine which cluster or set of nodes needs to be updated during this script run
	clusterName = System.getProperty("bpm.upgrade.cluster.name")
	nodeNames = None
	if not clusterName:
		nodeNames = [ System.getProperty("local.node") ]
		managedNodes = System.getProperty("bpm.upgrade.managed.nodes")[1:-1]
		if managedNodes:
			nodeNames.extend(managedNodes.split())
		#endif
	#endif

	# iterate over all supplied applications
	for appName in appNames:
		deployment = AdminConfig.getid("/Deployment:" + appName + "/")
		# iterate over all deployment targets of the current application
		for deploymentTarget in wsadminToList(AdminConfig.showAttribute(deployment, "deploymentTargets")):
			# convert 'deploymentTarget' into a map containing either 'name' and 'nodeName' keys (server) or a 'name' key (cluster)
			nlist = AdminConfig.show(deploymentTarget).split("\n")
			ndict = {}
			for i in range(len(nlist)):
				key, value = wsadminToList(nlist[i].rstrip())
				ndict[key] = value
			#endfor
			if clusterName:
				# if we are updating a given cluster, check if the current deployment target is that cluster
				if not ndict.has_key("nodeName") and ndict["name"] == clusterName:
					updateDeploymentTarget(appName, None, None, clusterName)
				#endif
			else:
				# if we are updating a set of nodes, check if the current deployment target is a server and it's on one of those nodes
				if ndict.has_key("nodeName") and ndict["nodeName"] in nodeNames:
					updateDeploymentTarget(appName, ndict["nodeName"], ndict["name"], None)
				#endif
			#endif
		#endfor
	#endfor
#enddef

#*******************************************************************************************************
# Run a single component's upgrade script
#*******************************************************************************************************
def runComponentUpgrade ( script ):
	global numscripts
	global numerrors
	import os
	rc = 0
	if ( not os.path.isfile( script ) ):
		framework.log("info", "BPMProfileUpgrade", "runComponentUpgrade",  "Script '" + script + "' not found, skipping.")
	#endIf
	else:
		framework.log("info", "BPMProfileUpgrade", "runComponentUpgrade", "Calling '" + script + "'...")
		numscripts = numscripts+1
		try:
			_excp_ = 0
			# copy the globals so scripts don't overwite any of our stuff
			error = execfile(script, globals().copy())
			_excp_ = 0 #reset (in case of nested exceptions)
		#endTry
		except:
			_type_, _value_, _tbck_ = sys.exc_info()
			error = str(_value_)
			_excp_ = 1
		#endTry
		# ignore error if return code was zero
		# this is what happens if system.exit(0) is called from script
		if (_excp_ == 1):
			if (error == '0'):
				rc = 0
			#endIf
			else:
				rc = _excp_
			#endElse
			#endIf
		else:
			rc = _excp_
			#endElse
		if (rc == 0):
			framework.log("info", "BPMProfileUpgrade", "runComponentUpgrade", "Done calling '" + script + "'.")
		#endIf
		else:
			framework.log("error", "BPMProfileUpgrade", "runComponentUpgrade", "Error calling '" + script + "': "+`error`+".")
			print str(_type_)
			print str(_value_)
			print str(_tbck_)
			numerrors = numerrors+1
		#endElse
	#endElse
	return rc
#endDef

class BPMMigrationFramework:
	def log(self, level, script, method, message):
		if (level == "debug"):
			MigrationLogger.logp(Level.FINE, script, method, message)
		if (level == "info"):
			MigrationLogger.logp(Level.INFO, script, method, message)
		if (level == "warning"):
			MigrationLogger.logp(Level.WARNING, script, method, message)
		if (level == "error"):
			MigrationLogger.logp(Level.SEVERE, script, method, message)
		if (level == "critical"):
			MigrationLogger.logp(Level.SEVERE, script, method, message)
		print message
	#enddef

#*******************************************************************************************************
# main
#*******************************************************************************************************
errorCode = ""
errorInfo = ""
numscripts = 0
numerrors = 0
runScript = ""
fileName = ""
framework = BPMMigrationFramework()
print "*************************************************************************************"
print "Operating system: " + System.getProperty("os.name")
print "WebSphere installation directory: " + System.getProperty("was.install.root")
print "WebSphere repository location: " + System.getProperty("was.repository.root")
print "Profile directory: " + System.getProperty("user.install.root")
print "Connection status: " + getConnectionStatus()

# extract arguments
i = 0  #forStart
while ( i < len(sys.argv) ):  #forTest
	arg = sys.argv[i]
	if (arg == "-migrationDir"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.backup.dir", arg)
		System.setProperty("bpm.upgrade.backup.dir", arg)
	#endIf
	if (arg == "-cluster"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.cluster.name", arg)
		System.setProperty("bpm.upgrade.cluster.name", arg)
	#endIf
	if (arg == "-filename"):
		i = i+1
		fileName = sys.argv[i]
	#endIf
	if (arg == "-bpm.upgrade.managed.nodes"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.managed.nodes", arg)
		System.setProperty("bpm.upgrade.managed.nodes", arg)
	#endIf
	if (arg == "-runScript"):
		i = i+1
		runScript = sys.argv[i]
		print "runScript arg found - " + runScript
	#endIf
	if (arg == "-wbi.upgrade.wsfep.source.enabled"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.wsfep.source.enabled", arg)
		System.setProperty("bpm.upgrade.wsfep.source.enabled", arg)
	#endIf
	if (arg == "-wbi.upgrade.wsfep.target.enabled"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.wsfep.target.enabled", arg)
		System.setProperty("bpm.upgrade.wsfep.target.enabled", arg)
	#endIf
	if (arg == "-wbi.prepare.migration"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.prepare.migration", arg)
	#endIf
	if (arg == "-bpm.upgrade.previous.profileName"):
		i = i+1
		arg = sys.argv[i]
		if (arg == "{}"):
			continue
		System.setProperty("wbi.upgrade.previous.profileName", arg)
		System.setProperty("bpm.upgrade.previous.profileName", arg)
	#endIf
	if (arg == "-bpm.upgrade.previous.hostName"):
		i = i+1
		arg = sys.argv[i]
		if (arg == "{}"):
			continue
		System.setProperty("bpm.upgrade.previous.hostName", arg)
	#endIf
	if (arg == "-bpm.upgrade.current.hostName"):
		i = i+1
		arg = sys.argv[i]
		if (arg == "{}"):
			continue
		System.setProperty("bpm.upgrade.current.hostName", arg)
	#endIf
	if (arg == "-wbi.upgrade.preserve.file"):
		i = i+1
		arg = sys.argv[i]
		System.setProperty("wbi.upgrade.preserve.file", arg)
	#endIf
	i += 1  #forNext
#endWhile  (#endFor)

if (System.getProperty("bpm.upgrade.cluster.name") and System.getProperty("bpm.upgrade.backup.dir")):
	print "Command line options: BPMMigrateCluster" + `sys.argv`
else:
	print "Command line options: BPMProfileUpgrade.ant" + `sys.argv`
#endIf

print "Initializing logger"
MigrationLogger.initialize(fileName, Boolean.TRUE)


if (connectionStatus == "ERROR"):
	framework.log("error", "BPMProfileUpgrade", "main", "Error: Required connection could not be established, aborting.")
	numerrors = 1
elif (checkClusterName(System.getProperty("bpm.upgrade.cluster.name")) == 0):
	framework.log("error", "BPMProfileUpgrade", "main", "Error: Specified cluster " + System.getProperty("bpm.upgrade.cluster.name") + " does not exist, aborting.")
	numerrors = 1
else:
	# set common variables for the component-specific scripts
	helper = WBIProfileUpgradeHelper(System.getProperty("was.install.root"), System.getProperty("was.repository.root"), System.getProperty("bpm.upgrade.backup.dir"))
	System.setProperty("wbi.upgrade.node.type", helper.getNodeType() )
	System.setProperty("bpm.upgrade.node.type", helper.getNodeType() )
	System.setProperty("bpm.upgrade.node.name", helper.getNodeName() )
	System.setProperty("wbi.upgrade.previous.was.version", helper.getWASPreviousVersion())
	System.setProperty("bpm.upgrade.previous.was.version", helper.getWASPreviousVersion())
	System.setProperty("wbi.upgrade.previous.version", helper.getWBIPreviousVersion())
	System.setProperty("bpm.upgrade.previous.version", helper.getBPMPreviousVersion("WBI"))
	System.setProperty("esb.upgrade.previous.version", helper.getBPMPreviousVersion("ESB"))
	System.setProperty("wbm.upgrade.previous.version", helper.getBPMPreviousVersion("WBM"))
	System.setProperty("wbps.upgrade.previous.version", helper.getBPMPreviousVersion("WBPS"))
	System.setProperty("wbi.upgrade.profile.type", helper.getProfileType())
	System.setProperty("bpm.upgrade.profile.type", helper.getBPMProfileType())
	if (System.getProperty("bpm.upgrade.previous.hostName")):
		framework.log("info", "BPMProfileUpgrade", "main", "Previous host name: " + str(System.getProperty("bpm.upgrade.previous.hostName")))
	if (System.getProperty("bpm.upgrade.current.hostName")):
		framework.log("info", "BPMProfileUpgrade", "main", "Current host name: " + str(System.getProperty("bpm.upgrade.current.hostName")))
	framework.log("info", "BPMProfileUpgrade", "main", "Node type: " + System.getProperty("bpm.upgrade.node.type"))
	framework.log("info", "BPMProfileUpgrade", "main", "Node name: " + System.getProperty("bpm.upgrade.node.name"))
	framework.log("info", "BPMProfileUpgrade", "main", "Previous node version: " + System.getProperty("wbi.upgrade.previous.version"))
	framework.log("info", "BPMProfileUpgrade", "main", "Profile type: " + System.getProperty("wbi.upgrade.profile.type"))
	framework.log("info", "BPMProfileUpgrade", "main", "Migration backup directory: " + str(System.getProperty("bpm.upgrade.backup.dir")))
	framework.log("info", "BPMProfileUpgrade", "main", "Previous profile name: " + str(System.getProperty("bpm.upgrade.previous.profileName")))
	framework.log("info", "BPMProfileUpgrade", "main", "Executing ProcessApplicationInformation.py")
	if (System.getProperty("bpm.upgrade.backup.dir")):
		runComponentUpgrade(System.getProperty("was.install.root") + "/util/upgrade/ProcessApplicationInformation.py")
		if (numerrors > 0):
			if (System.getProperty("bpm.upgrade.cluster.name")):
				if (System.getProperty("bpm.upgrade.backup.dir")):
					data = helper.updateConfigStatus("Failed", System.getProperty("bpm.upgrade.cluster.name"), fileName);
					framework.log("error", "BPMMigrateCluster", "main", "Script ProcessApplicationInformation.py failed, did not complete successfully " + numerrors)
					sys.exit(numerrors)
				#endIf
			#endIf
 		#endIf
	#endIf
	if (len(runScript) > 0):
		framework.log("info", "BPMProfileUpgrade", "main", "runScript: " + runScript)
		runComponentUpgrade(runScript)
	else:
		# invoke the component-specific scripts
		script = helper.getNextJythonScript()
		while (len(script) > 0):
			runComponentUpgrade(script )
			if (numerrors > 0):
				break
			#endIf
			script = helper.getNextJythonScript()
		#endWhile
	#endElse

	if (System.getProperty("bpm.upgrade.cluster.name")):
		if (System.getProperty("bpm.upgrade.backup.dir")):
 			if (numerrors > 0):
				data = helper.updateConfigStatus("Failed", System.getProperty("bpm.upgrade.cluster.name"), fileName);
				framework.log("error", "BPMMigrateCluster", "main", "Script failed, did not complete successfully ")
			else:
				data = helper.updateConfigStatus("Success", System.getProperty("bpm.upgrade.cluster.name"), fileName);
				framework.log("info", "BPMMigrateCluster", "main", "Script completed successfully")
				#Stamp the cluster
				System.setProperty("bpm.upgrade.cluster.stamp", "true")
				runComponentUpgrade(System.getProperty("was.install.root") + "/util/upgrade/MigrationCommon.py")
			#endElse
		#endIf
	#endIf
#endElse
sys.exit(numerrors)
